# Program Corpus

This directory contains a corpus of trajectories drawn by real people, along with Turtle programs they wrote to reproduce their drawings, and the sequence of editing commands they used to write each program. The dataset was originally purposed to evaluate the synthesis algorithms.

Citation: Evan Hernandez, Ara Vartanian, and Xiaojin Zhu. Program synthesis with visual specification. ArXiv 1806.00938, 2018.

## Organization

The data is organized as follows. There are 23 trials and each trial contains three files, the contents of which are summarized in the table below. The files for trial _n_ share the common prefix of _n_ followed by an underscore. For example, all files for trial 1 take the form `01_<file name>`.


| File              	| Description                                                                                                            	|
|-------------------	|------------------------------------------------------------------------------------------------------------------------	|
| `drawing.png`     	| Participant's trajectory, drawn in, e.g., MS Paint or Mac OSX Paintbrush, with a 1px white pen on a standard canvas.                                 	|
| `program.txt`     	| Program written by the participant to reproduce their trajectory.                                                         	|
| `commands.txt`       	| The sequence of editing commands used by participant to write their program. This file may include comments, which are preceded by a % symbol.                              	|


## Data Collection

What follows is a high-level overview of the process used to construct this corpus.

### Step 1: Participant Draws Intended Trajectory

Participants are first asked to draw a trajectory on a standard canvas, shown in  [`canvas.png`](canvas.png). They may use any painting tool with a 1px white pen. Importantly, participants must  start drawing at the center of the canvas.

### Step 2: Participant Writes Program

The participant is then asked to write a program that reproduces their hand-drawn trajectory.

Participants write their programs using a hacked version of [Blockly Turtle](https://blockly-games.appspot.com/turtle?lang=en). The interface is modified so that each block was labeled with a unique ID. This allows the user's mouse actions to be recorded as editing commands. The mapping between mouse actions and editing commands is described in the paper.

### Step 3: Final Program Recorded

The resulting program is recorded using a textual representation of the Turtle language. It uses three keywords:
- `move`
- `turn`
- `repeat`

The `turn` statement takes the (integral) number of degrees to turn as a parameter. The `repeat` statement takes the (integral) number of times to repeat, followed by the program block to repeat surrounded by curly brackets {}. Each statement is separated by a newline, and by convention all keywords are lowercase.

Here is a sample program:

```
repeat 4 {
  move
  turn 90
}
move
```
Technically, the final program is redundant because it may be recovered by executing all of a participant's editing commands in sequence. Regardless, the final programs are provided here for convenience and clarity.
